unit UMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, EditBtn,
  ExtCtrls, LazFileUtils, FPimage, FPReadBMP, FPReadGif, FPReadJPEG, FPReadPCX,
  FPReadPNG, FPReadPNM, FPReadPSD, FPReadTGA, FPReadTiff, FPReadXPM, FPReadXWD,
  FPCanvas, FPImgCanv;

type

  { TEntry }

  TEntry = record
    FileName: string;
    BaseName: string;
    Comment: string;
    Keywords: array of string;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Label1: TLabel;
    fneGalleryFile: TFileNameEdit;
    Label2: TLabel;
    deOutput: TDirectoryEdit;
    btCreateGallery: TButton;
    mStatus: TMemo;
    Bevel1: TBevel;
    procedure btCreateGalleryClick(Sender: TObject);
    procedure fneGalleryFileAcceptFileName(Sender: TObject; var Value: String);
  private
    Entries: array of TEntry;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.btCreateGalleryClick(Sender: TObject);
var
  Path, Output: string;
  Lines: TStringList;
  I, J, K: Integer;
  Parts: TStringArray;
  Jpg: TJPEGImage;
  Img, SmallImg: TFPMemoryImage;
  SICanvas: TFPImageCanvas;
  AllKeywords: TStringArray;
  Found: Boolean;

  function Dequote(S: string): string;
  begin
    if S[1]='"' then Exit(Copy(S, 2, Length(S) - 2)) else Exit(S);
  end;

  procedure CreateGalleryFile(ATitle, AFileName: string; ATag: string);
  var
    Code: TStringList;
    I, J, Cnt, FullCount: Integer;
    Ok: Boolean;
  begin
    Code:=TStringList.Create;
    Code.Add('<HTML><HEAD><TITLE>' + ATitle + '</TITLE></HEAD>');
    Code.Add('<BODY BGCOLOR=#000000 TEXT=#FFFFFF LINK=#FF0000 ALINK=#FF0000 VLINK=#FF0000>');
    Code.Add('<H1>' + ATitle + '</H1><HR>');
    Code.Add('<CENTER><TABLE BORDER=0><TR VALIGN=TOP>');
    Cnt:=-1;
    FullCount:=0;
    for I:=0 to High(Entries) do with Entries[I] do begin
      if ATag <> '' then begin
        Ok:=False;
        for J:=0 to High(Keywords) do
          if Keywords[J]=ATag then begin
            Ok:=True;
            Break;
          end;
        if not Ok then Continue;
      end;
      Inc(Cnt);
      if Cnt=4 then begin
        Code.Add('</TR><TR VALIGN=TOP>');
        Cnt:=0;
      end;
      Code.Add('<TD><CENTER><A HREF="' + BaseName + '.jpg"><IMG BORDER=0 SRC="tmb_' + BaseName + '.jpg" WIDTH=240 HEIGHT=240><BR>');
      Code.Add(BaseName + '</A></CENTER>');
      if Comment <> '' then Code.Add('<BR><I>' + StringReplace(StringReplace(Comment, '<', '&lt', [rfReplaceAll]), '&', '&amp;', [rfReplaceAll]) + '</I>');
      if Length(Keywords) > 0 then begin
        Code.Add('<BR>Keywords:');
        for J:=0 to High(Keywords) do
          Code.Add('<A HREF="key_' + Keywords[J] + '.html">' + Keywords[J] + '</A>');
        Code.Add(' &nbsp;&nbsp');
      end;
      Code.Add('</TD>');
      Inc(FullCount);
    end;
    for Cnt:=Cnt + 1 to 4 do Code.Add('<TD>&nbsp;</TD>');
    Code.Add('</TR></TABLE></CENTER><HR><B>' + IntToStr(FullCount) + ' images</B></BODY></HTML>');
    Code.SaveToFile(AFileName);
    Code.Free;
  end;

begin
  mStatus.Clear;
  Path:=Trim(fneGalleryFile.FileName);
  if Path='' then begin
    ShowMessage('Select a gallery file first');
    fneGalleryFile.SetFocus;
    Exit;
  end;
  if not FileExists(Path) then begin
    ShowMessage('The file "' + Path + '" does not exist.');
    Exit;
  end;
  Output:=Trim(deOutput.Directory);
  if Output='' then begin
    ShowMessage('Select an output directory');
    deOutput.SetFocus;
    Exit;
  end;
  Lines:=TStringList.Create;
  try
    Lines.LoadFromFile(Path);
  except
    ShowMessage('Failed to load "' + Path + '"');
    Lines.Free;
    Exit;
  end;
  SetLength(Entries, Lines.Count);
  for I:=0 to Lines.Count - 1 do with Entries[I] do begin
    Parts:=Lines[I].Split([','], '"', '"');
    FileName:=Dequote(Parts[0]);
    BaseName:=ExtractFileNameWithoutExt(ExtractFileName(FileName));
    Comment:=Dequote(Parts[1]);
    SetLength(Keywords, Length(Parts) - 2);
    for J:=2 to High(Parts) do
      Keywords[J - 2]:=LowerCase(Dequote(Parts[J]));
  end;
  Lines.Free;
  if not ForceDirectories(Output) then begin
    Writeln('Failed to create directory "' + Output + '"');
    Exit;
  end;
  AllKeywords:=nil;
  for I:=0 to High(Entries) do with Entries[I] do begin
    for J:=0 to High(Keywords) do begin
      Found:=False;
      for K:=0 to High(AllKeywords) do
        if AllKeywords[K]=Keywords[J] then begin
          Found:=True;
          Break;
        end;
      if not Found then begin
        SetLength(AllKeywords, Length(AllKeywords) + 1);
        AllKeywords[High(AllKeywords)]:=Keywords[J];
      end;
    end;
    mStatus.Lines.Add('Converting ' + BaseName);
    Img:=TFPMemoryImage.Create(1, 1);
    SmallImg:=TFPMemoryImage.Create(240, 240);
    SICanvas:=TFPImageCanvas.Create(SmallImg);
    try
      Img.LoadFromFile(FileName);
      if Img.Width > Img.Height then begin
        J:=Round((Img.Height/Img.Width)*240);
        SICanvas.StretchDraw(0, 120 - J div 2, 240, J, Img);
      end else begin
        J:=Round((Img.Width/Img.Height)*240);
        SICanvas.StretchDraw(120 - J div 2, 0, J, 240, Img);
      end;
      Jpg:=TJPEGImage.Create;
      Jpg.Assign(SmallImg);
      Jpg.CompressionQuality:=65;
      Jpg.SaveToFile(Output + '/tmb_' + BaseName + '.jpg');
      Jpg.Free;
      Jpg:=TJPEGImage.Create;
      Jpg.Assign(Img);
      Jpg.CompressionQuality:=90;
      Jpg.SaveToFile(Output + '/' + BaseName + '.jpg');
      Jpg.Free;
    except
      mStatus.Lines.Add('...failed to open the file');
    end;
    SICanvas.Free;
    SmallImg.Free;
    Img.Free;
  end;
  mStatus.Lines.Add('Creating index.html');
  CreateGalleryFile('Full Gallery', Output + '/index.html', '');
  for I:=0 to High(AllKeywords) do begin
    if AllKeywords[I]='' then Continue;
    mStatus.Lines.Add('Creating key_' + AllKeywords[I] + '.html');
    CreateGalleryFile('Images for keyword: ' + AllKeywords[I], Output + '/key_' + AllKeywords[I] + '.html', AllKeywords[I]);
  end;
end;

procedure TForm1.fneGalleryFileAcceptFileName(Sender: TObject; var Value: String);
begin
  if deOutput.Directory='' then deOutput.Directory:=ExtractFileDir(Value) + DirectorySeparator + 'output';
end;

end.

